local addon_name, addon = ...
local tt = addon.tooltip

local token = addon.token
local util = addon.utility
local time = addon.time
local global = addon.global
local _G = _G

local tooltip
local realm = GetRealmName()
local TTGap = 10

local Optional = {}
Optional.ShowDefault = {}
Optional.DispFunc = {}
Optional.LenFunc = {}
Optional.DispName = {}
Optional.CalcAvg = {}
Optional.TimeBased = {}
Optional.SpaceAfter = {}
Optional.MenuSeperator = {}
Optional.Keys = {}
local function RegisterOptionalSetting(key, dispName, defaultShow, dispFunc, lenFunc, calcAvg, spaceAfter, menuSeperator, timeBased)
   if key == nil or type(key) ~= "string" then 
      error("key should be a string")
      return
   end

   if dispName == nil or type(dispName) ~= "string" then 
      error("dispName should be a string")
      return
   end

   if defaultShow == nil or type(defaultShow) ~= "boolean" then 
      error("defaultShow should be a boolean")
      return
   end

   if dispFunc == nil or type(dispFunc) ~= "function" then 
      if dispFunc then
         error("dispFunc should be a function")
         return
      end
   end

   if lenFunc == nil or type(lenFunc) ~= "function" then 
      if lenFunc then
         error("lenFunc should be a function")
         return
      end
   end

   if calcAvg == nil then
      calcAvg = false
   end

   if type(calcAvg) ~= "boolean" then 
      error("calcAvg should be a boolean")
      return
   end

   if type(timeBased) ~= "boolean" then 
      error("timeBased should be a boolean")
      return
   end

   if type(spaceAfter) ~= "boolean" then 
      error("spaceAfter should be a boolean")
      return
   end

   if type(menuSeperator) ~= "boolean" then 
      error("menuSeperator should be a boolean")
      return
   end

   if not Optional.DispFunc[key] then
      Optional.DispName[key] = dispName
      Optional.ShowDefault[key] = defaultShow
      Optional.DispFunc[key] = dispFunc
      Optional.LenFunc[key] = lenFunc
      Optional.CalcAvg[key] = calcAvg
      Optional.TimeBased[key] = timeBased
      Optional.SpaceAfter[key] = spaceAfter
      Optional.MenuSeperator[key] = menuSeperator
      table.insert(Optional.Keys, key)
   end
end

local function GetOptionalSettingShown(key)
   local shown = _G.WOWMMChar[key.."Shown"]
   if shown == nil then
      shown = Optional.ShowDefault[key]
   end
   return shown
end

local function AddCashItem(title, cash, pos, class)
   local r, g, b = 1, 1, 1
   if class then
      r, g, b = util.GetClassRGB(class)
   end
   if pos then
      tooltip:AddDoubleLine(title, util.FormatMoney(cash), r, g, b, 1, 1, 1)
   else
      tooltip:AddDoubleLine(title, util.FormatMoney(cash), r, g, b, 1, 0, 0)
   end
end

local function AddSeperator()
   local total = 0
   local longestLeft = "Character:"

   for name, value in pairs(_G.WOWMMGlobal) do
      if type(value) == "table" then
         total = total + value.Total
      end
   end

   for name, _ in pairs(_G.WOWMMGlobal[realm].Chars) do
      if #name > #longestLeft then
         longestLeft = name
      end
   end

   for name, v in pairs(_G.WOWMMGlobal) do
      if type(v) == "table" then
         if #name > #longestLeft then
            longestLeft = name
         end
      end
   end

   local longestRight = util.FormatMoney(total)

   for i = 1, #Optional.Keys do
      local key = Optional.Keys[i]
      if GetOptionalSettingShown(key) then
         if Optional.LenFunc[key] then
            local left, right = Optional.LenFunc[key]()
            if #left > #longestLeft then
               longestLeft = left
            end
            if #right > #longestRight then
               longestRight = right
            end
         end
      end
   end

   tooltip:AddSeperator("-", longestLeft, longestRight, TTGap)
end

local function GetAvg(key)
   if _G.WOWMMGlobal[realm][key.."Count"] > 0 then
      local A = floor(_G.WOWMMGlobal[realm][key.."Net"] / _G.WOWMMGlobal[realm][key.."Count"])
      return A
   else
      local E = _G.WOWMMGlobal[realm][key].Earned
      local S = _G.WOWMMGlobal[realm][key].Spent
      return (E - S)
   end
end

local function PopulateTooltip()
   local TimeBasedShown = false

   for i = 1, #Optional.Keys do
      local key = Optional.Keys[i]
      if GetOptionalSettingShown(key) then
         TimeBasedShown = Optional.TimeBased[key] or TimeBasedShown
         if Optional.DispFunc[key] then
            Optional.DispFunc[key]()
         end
         if Optional.SpaceAfter[key] then
            tooltip:AddLine(' ')
         end
      end
   end

   if TimeBasedShown then
      AddSeperator()
      tooltip:AddLine(' ')
   end
   tooltip:AddLine("Character:")
   local chars = util.GetSortedCharList(realm)
   for _, name in pairs(chars) do
      AddCashItem(name, _G.WOWMMGlobal[realm].Chars[name].Cash, true, _G.WOWMMGlobal[realm].Classes[name])
   end
   tooltip:AddLine(' ')
   tooltip:AddLine("Server:")

   local total = 0
   for name, value in pairs(_G.WOWMMGlobal) do
      if type(value) == "table" then
         total = total + value.Total
         AddCashItem(name, value.Total, true)
      end
   end
   tooltip:AddLine(' ')
   AddCashItem("Total:", total, true)
end

function tt.ShowTooltip(self)
   tooltip:Init(self)
   PopulateTooltip()
   tooltip:Show()
end

function tt.HideTooltip()
   tooltip:Hide()
end

local function SetFont(font)
   tooltip:SetFont(font)
end

local function GetFont()
   tooltip:GetFont()
end

local function SetFontSize(fontSize)
   tooltip:SetFontSize(fontSize)
end

local function GetFontSize()
   tooltip:GetFontSize()
end

local function UpdateFontInfo()
   SetFont(util.GetFontSetting())
   SetFontSize(util.GetFontSizeSetting())
end

function tt.OptionalSettingMenu(menu, level)
   for i = 1, #Optional.Keys do
      local key = Optional.Keys[i]
      local shown = key.."Shown"
      local text = "Show "..Optional.DispName[key]
      menu:AddCheckOption( text,
                              function(self)
                                 return tt.GetOptionalSettingShown(key)
                              end,
                              level,
                              function(self, _, _, checked)
                                 _G.WOWMMChar[shown] = checked
                              end
                           )
      if Optional.MenuSeperator[key] then
         menu:AddDescriptor    (  "-------------------", level )
      end
   end
end

tt.GetOptionalSettingShown = GetOptionalSettingShown
tt.UpdateFontInfo = UpdateFontInfo

tooltip = Prototype_Tooltip:new()

RegisterOptionalSetting ("WowTokenPrice",
                           "WoW Token Price",
                           false,
                           function()
                              if C_WowTokenPublic.GetCommerceSystemStatus() then
                                 AddCashItem("WoW Token Price:", token.GetTokenPrice(), true)
                              else
                                 tooltip:AddDoubleLine("WoW Token Price:", "System Offline", 1, 1, 1, 1, 1, 1)
                              end
                              tooltip:AddLine(' ')
                              AddSeperator()
                           end,
                           function()
                              return "WoW Token Price:", util.FormatMoney(token.GetTokenPrice())
                           end,
                           false,
                           true,
                           false,
                           false
                        )
RegisterOptionalSetting ("WatchedCurrencies",
                           "Watched Currencies",
                           false,
                           function()
                              local currencyCount = GetCurrencyListSize()
                              local shownCount = 0
                              for i = 1, currencyCount do
                                 local currency, isHeader, _, isUnused, isWatched, count, icon = GetCurrencyListInfo(i)
                                 if not isHeader and not isUnused and isWatched then
                                    shownCount = shownCount + 1
                                    tooltip:AddDoubleLine(util.FormatCurrency(currency..":", icon), count, 1, 1, 1, 1, 1, 1)
                                 end
                              end
                              if shownCount ~= 0 then
                                 tooltip:AddLine(' ')
                                 AddSeperator()
                                 tooltip:AddLine(' ')
                              end
                           end,
                           function()
                              local currencyCount = GetCurrencyListSize()
                              local left = ""
                              local right = ""
                              for i = 1, currencyCount do
                                 local currency, isHeader, _, isUnused, isWatched, count, icon = GetCurrencyListInfo(i)
                                 if not isHeader and not isUnused and isWatched then
                                    local formattedLeft = util.FormatCurrency(currency..":", icon)
                                    if #left < #formattedLeft then
                                       left = formattedLeft
                                    end
                                    count = format("%d", count)
                                    if #right < #count then
                                       right = count
                                    end
                                 end
                              end
                              return left, right
                           end,
                           false,
                           false,
                           true,
                           false
                        )

-- Register all Time based Optional Settings
for _, v in pairs(time.TimeList) do
   RegisterOptionalSetting (  v,                                                          -- Persistance Key
                              time.TimeSetTitles[v],                                      -- Display Title
                              true,                                                       -- Default Show
                              function()                                                  -- Display Function
                                 local E = _G.WOWMMGlobal[realm][v].Earned
                                 local S = _G.WOWMMGlobal[realm][v].Spent
                                 tooltip:AddLine(format("%s:", time.TimeSetTitles[v]))
                                 AddCashItem("Earned:", E, true)
                                 AddCashItem("Spent:", S, (S == 0))
                                 AddCashItem("Net:", (E - S), E >= S)
                                 if util.getSetValue(time.TimeSetAvg, v) then
                                    if Optional.CalcAvg[v] and GetOptionalSettingShown("Avg") then
                                       local avg = GetAvg(v)
                                       AddCashItem("Avg:", avg, avg >= 0, "EMPHASIS")
                                    end
                                 end
                              end,
                              function()                                                  -- Length Function
                                 local left = format("%s:", time.TimeSetTitles[v])
                                 local right = ""
                                 
                                 if #left < #("Earned:") then
                                    left = "Earned:"
                                 end

                                 local E = _G.WOWMMGlobal[realm][v].Earned
                                 local S = _G.WOWMMGlobal[realm][v].Spent
                                 local earned = util.FormatMoney(E)
                                 local spent = util.FormatMoney(S)
                                 local net = util.FormatMoney(E-S)
                                 local avg = ""
                                 if util.getSetValue(time.TimeSetAvg, v) then
                                    avg = util.FormatMoney(GetAvg(v))
                                 end
                                 if #right < #earned then
                                    right = earned
                                 end
                                 if #right < #spent then
                                    right = spent
                                 end
                                 if #right < #net then
                                    right = net
                                 end
                                 if #right < #avg then
                                    right = avg
                                 end
                                 return left, right
                              end,
                              util.getSetValue(time.TimeSetAvg, v),                       -- Calculate Average
                              true,                                                       -- Space After Display
                              (v == "Session"),                                           -- Seperator After in Menu
                              true                                                        -- Time Based Optional Setting
                           )


end
RegisterOptionalSetting ("Avg",
                           "Averages",
                           true,
                           nil, -- Avg Display is handled internally in other options
                           nil, -- Avg Length is handled internally in other options
                           false,
                           false,
                           false,
                           false
                        )
local frame = CreateFrame("Frame")

local function EventHandler(self, event, ...)
   local args = {...}
   if event == "ADDON_LOADED" and args[1] == addon_name then
      global.InitGlobals()

      UpdateFontInfo()
   end
end

util.RegisterEvents(frame, EventHandler, 'ADDON_LOADED')
