local addon_name, addon = ...
local mt = addon.mt

local util = addon.utility
local sd = addon.statDisplay
local _G = _G

local player = UnitName("player")
local realm = GetRealmName()
--local guildName = GetGuildInfo("player")

local MainTracker = CreateFrame("Frame")
MainTracker.UnprocessedMoney = false

local PurgeTimer = CreateFrame("Frame")
PurgeTimer.Elapsed = 0
PurgeTimer.TimeToPurge = 2

local ActiveTrackers = 
{
   ActiveCount = 0,
   Trackers = {}
}

mt.DisplayNames =
{
   TAXI                 = "Taxi",
   TRADE                = "Trade",
   TRAINER              = "Trainer",
   AUCTION              = "Auction",
   BMAH                 = "Black Market",
   ARCHITECT            = "Garrison Arch",
   MISSION              = "Mission Table",
   SHIPYARD             = "Shipyard",
   TRANSMOG             = "Transmog",
   VOID                 = "Void Storage",
   GUILDBANK            = "Guild",
   LOOT                 = "Loot",
   REPAIR               = "Repair",
   MERCH                = "Vendor",
   COD                  = "C.O.D.",
   ALT                  = "Alt",
   POSTAGE              = "Postage",
   QUEST                = "Quest",
   AZERITEREFORGE       = "Reforge",
}

local RegisteredTrackers = {}
RegisteredTrackers.ActivationFunction = {}
RegisteredTrackers.OnTriggerFunction = {}
RegisteredTrackers.OnActivationFunction = {}
RegisteredTrackers.DeactivationFunction = {}
RegisteredTrackers.OnDeactivationFunction = {}
RegisteredTrackers.DeterminantFunction = {}
RegisteredTrackers.Keys = {}

local function isTrackerActive(uuid)
   local trackerActive = false
   for k, _ in pairs(ActiveTrackers.Trackers) do
      trackerActive = trackerActive or (uuid == k)
   end

   return trackerActive
end

local function ActivateTracker(uuid, t, ...)
   if not ActiveTrackers.Trackers[uuid] then
      ActiveTrackers.ActiveCount = ActiveTrackers.ActiveCount + 1
      ActiveTrackers.Trackers[uuid] =
      {
         Tracker = t,
         Money = 0,
         DeterminantTriggered = false,
         Catagorized = false
      }
   end

   -- Trackers with Determinants are only available when the determinant gets set
   if RegisteredTrackers.DeterminantFunction[uuid] then
      ActiveTrackers.Trackers[uuid].Catagorized = true
   end

   -- On Activation
   if RegisteredTrackers.OnActivationFunction[uuid] then
      RegisteredTrackers.OnActivationFunction[uuid](...)
   end
end

local function Purge(self, elapsed)
   self.Elapsed = self.Elapsed + elapsed
   if self.Elapsed >= self.TimeToPurge then
      ActiveTrackers.LastDeactivatedTracker = nil
      self.Elapsed = 0
      self:SetScript("OnUpdate", nil)
   end
end

local function DeactivateTracker(uuid, ...)
   -- On Deactivation
   if RegisteredTrackers.OnDeactivationFunction[uuid] then
      RegisteredTrackers.OnDeactivationFunction[uuid](...)
   end

   if ActiveTrackers.Trackers[uuid] then
      ActiveTrackers.LastDeactivatedTracker = 
      {
         UUID = uuid,
         Tracker = ActiveTrackers.Trackers[uuid].Tracker,
         Money = ActiveTrackers.Trackers[uuid].Money,
      }
      ActiveTrackers.ActiveCount = ActiveTrackers.ActiveCount - 1
      ActiveTrackers.Trackers[uuid] = nil

      -- Start timer for cleanup
      PurgeTimer.Elapsed = 0
      PurgeTimer:SetScript("OnUpdate", Purge)
   end
end

local function SetDeterminant(uuid, money)
   if not uuid then
      return
   end

   if ActiveTrackers.Trackers[uuid] then
      ActiveTrackers.Trackers[uuid].DeterminantTriggered = true
      ActiveTrackers.Trackers[uuid].Catagorized = false
      ActiveTrackers.Trackers[uuid].Money = ActiveTrackers.Trackers[uuid].Money + money

      if MainTracker.UnprocessedMoney == true then
         MainTracker:DetermineSource(MainTracker.DeltaMoney)
      end
   end
end

local function Waiting(self)
   addon.debugPrint("Waiting", self.DeltaMoney)
   self:SetScript("OnUpdate", nil)
   self:DetermineSource(self.DeltaMoney)
end

local RetryCount = 0
function MainTracker:DetermineSource(money)
   addon.debugPrintTable(ActiveTrackers)
   addon.debugPrint(money)
   local catagorized = false

   local UnderSpecified
   for uuid, v in pairs(ActiveTrackers.Trackers) do
      local m, t
      if v.DeterminantTriggered and v.Catagorized == false then
         m = v.Money
         t = v.Tracker

         if m ~= 0 then
            money = money - m
            RegisteredTrackers.OnTriggerFunction[uuid](uuid, t, m)
            --print(format("%s catagorized as %s", util.FormatMoney(m), t))
            if money == 0 then
               catagorized = true
            end
            v.DeterminantTriggered = false
            v.Catagorized = true
            v.Money = 0
         else
            UnderSpecified = uuid
         end
      end
   end

   if UnderSpecified and money ~= 0 then
      local m, t, v, uuid
      uuid = UnderSpecified
      m = money
      v = ActiveTrackers.Trackers[uuid]
      t = v.Tracker
      catagorized = true
      v.DeterminantTriggered = false
      v.Catagorized = true
      RegisteredTrackers.OnTriggerFunction[uuid](uuid, t, m)
      --print(format("%s catagorized as %s", util.FormatMoney(m), t))
   end

   if catagorized == false and ActiveTrackers.LastDeactivatedTracker ~= nil then
      if ActiveTrackers.LastDeactivatedTracker.Money == 0 then
         ActiveTrackers.LastDeactivatedTracker.Money = money
      end
      local m = ActiveTrackers.LastDeactivatedTracker.Money
      local t = ActiveTrackers.LastDeactivatedTracker.Tracker
      local uuid = ActiveTrackers.LastDeactivatedTracker.UUID

      RegisteredTrackers.OnTriggerFunction[uuid](uuid, t, m)
      --print(format("%s catagorized as %s", util.FormatMoney(m), t))
      catagorized = true
   end

   if catagorized == false and ActiveTrackers.ActiveCount > 0 then
      local count = 0
      local uuid
      for k, v in pairs(ActiveTrackers.Trackers) do
         if v.Catagorized == false then
            uuid = k
            count = count + 1
         end
      end

      if count == 1 then
         local t = ActiveTrackers.Trackers[uuid].Tracker
         local m = money

         RegisteredTrackers.OnTriggerFunction[uuid](uuid, t, m)
         --print(format("%s catagorized as %s", util.FormatMoney(m), t))
         catagorized = true
      end
   end

   if MainTracker.UnprocessedMoney then
      addon.debugPrint("Retried!")
   end

   if catagorized == false then
      if ActiveTrackers.ActiveCount > 0 and RetryCount < 3 then
         addon.debugPrint("Retrying")
         RetryCount = RetryCount + 1
         MainTracker.UnprocessedMoney = true
      else
         RetryCount = 0
         MainTracker.UnprocessedMoney = false
         --print(format("%s catagorized as %s", util.FormatMoney(money), "UNKNOWN"))
      end
   else
      RetryCount = 0
      MainTracker.UnprocessedMoney = false
   end

   sd.UpdateStatDialog()
end

local function InitPersistance()
   _G.WOWMSTracker[realm] = _G.WOWMSTracker[realm] or {}
   _G.WOWMSTracker[realm][player] = _G.WOWMSTracker[realm][player] or {}

   _G.WOWMSTracker[realm].AllChars = _G.WOWMSTracker[realm].AllChars or {}

   for k, t in pairs(RegisteredTrackers.Keys) do
      _G.WOWMSTracker[realm].AllChars[t] = _G.WOWMSTracker[realm].AllChars[t] or {}
      _G.WOWMSTracker[realm].AllChars[t].Earned = _G.WOWMSTracker[realm].AllChars[t].Earned or 0
      _G.WOWMSTracker[realm].AllChars[t].Spent = _G.WOWMSTracker[realm].AllChars[t].Spent or 0

      _G.WOWMSTracker[realm][player][t] = _G.WOWMSTracker[realm][player][t] or {}
      _G.WOWMSTracker[realm][player][t].Earned = _G.WOWMSTracker[realm][player][t].Earned or 0
      _G.WOWMSTracker[realm][player][t].Spent = _G.WOWMSTracker[realm][player][t].Spent or 0
   end
end

local function EventHandler(self, event, ...)
   addon.debugPrint(event, ...)
   for k, _ in pairs(RegisteredTrackers.Keys) do
      RegisteredTrackers.ActivationFunction[k](event, ...)
      RegisteredTrackers.DeactivationFunction[k](event, ...)

      if RegisteredTrackers.DeterminantFunction[k] then
         for _, v in pairs(RegisteredTrackers.DeterminantFunction[k]) do
            v(event, ...)
         end
      end
   end

   if event == "PLAYER_ENTERING_WORLD" then
      MainTracker.Money = GetMoney()

      InitPersistance()
   end

   if event == "PLAYER_MONEY" then
      local money = GetMoney()
      MainTracker.DeltaMoney = money - MainTracker.Money
      MainTracker.Money = money
      MainTracker:SetScript("OnUpdate", Waiting)
   end
end

function mt.RegisterTracker(uuid, t, activationEvent, deactivationEvent, onTrigger)
   if not uuid or not t or not activationEvent or not deactivationEvent or not onTrigger then
      error("RegisterTracker failed - Not enough arguments")
   end

   -- Create Key
   if not util.setContains(RegisteredTrackers.Keys, uuid) then
      util.addToSet(RegisteredTrackers.Keys, uuid, t)
   end

   -- ActivationFunction
   if not RegisteredTrackers.ActivationFunction[uuid] then
      RegisteredTrackers.ActivationFunction[uuid] =
      function (event, ...)
         if event == activationEvent then
            ActivateTracker(uuid, t, ...)
         end
      end
   end

   -- DeactivationFunction
   if not RegisteredTrackers.DeactivationFunction[uuid] then
      RegisteredTrackers.DeactivationFunction[uuid] = 
      function(event, ...)
         if event == deactivationEvent then
            DeactivateTracker(uuid, ...)
         end
      end
   end

   -- OnInitialization
   if not RegisteredTrackers.OnTriggerFunction[uuid] then
      RegisteredTrackers.OnTriggerFunction[uuid] = onTrigger
   end

   -- Register Events
   util.RegisterEvents(MainTracker, EventHandler, activationEvent, deactivationEvent)
end

function mt.RegisterDeterminant(uuid, determinantEvent, classifier)
   if not uuid or not determinantEvent or not classifier then
      error("RegisterDeterminant failed - Not enough arguments")
   end

   -- DeterminantFunction
   if not RegisteredTrackers.DeterminantFunction[uuid] then
      RegisteredTrackers.DeterminantFunction[uuid] = {}
   end

   tinsert(RegisteredTrackers.DeterminantFunction[uuid],
   function(event, ...)
      if event == determinantEvent then
         if isTrackerActive(uuid) == true then
            SetDeterminant(classifier(uuid, ...))
         end
      end
   end)

   -- Register Events
   util.RegisterEvents(MainTracker, EventHandler, determinantEvent)
end

function mt.RegisterOnActivation(uuid, func)
   RegisteredTrackers.OnActivationFunction[uuid] = func
end

function mt.RegisterOnDeactivation(uuid, func)
   RegisteredTrackers.OnDeactivationFunction[uuid] = func
end

util.RegisterEvents(MainTracker, EventHandler, "PLAYER_MONEY")
util.RegisterEvents(MainTracker, EventHandler, "PLAYER_ENTERING_WORLD")

mt.InitPersistance = InitPersistance
