local _, addon = ...

local mt = addon.mt
local util = addon.utility
local _G = _G

local player = UnitName("player")
local realm = GetRealmName()

local MERCH_UUID = 'MERCH_01'    -- Profits or Expenses from a Merchant
local REPAIR_UUID = 'MERCH_02'   -- Expenses from Repairing armor

local money
local repairCost

local function OnActivation(...)
   money = GetMoney()
   if CanMerchantRepair() then
      repairCost = GetRepairAllCost()
   else
      repairCost = 0
   end
end

local function Classify(uuid)
   local rc, cr = GetRepairAllCost()
   --local crg = CanGuildBankRepair()
   local m = GetMoney()

   local TotalRepairCost = rc - repairCost
   local repaired = rc ~= repairCost

   if uuid == REPAIR_UUID and CanMerchantRepair() then
      if repaired == true and TotalRepairCost ~= 0 then
         repairCost = rc
         repaired = false
         money = money + TotalRepairCost
         return REPAIR_UUID, TotalRepairCost
      end
   end

   if uuid == MERCH_UUID then
      local cost = m - money
      if repaired and cost ~= 0 then
         cost = cost + TotalRepairCost
      end
      if cost ~= 0 then
         money = m
         return MERCH_UUID, cost
      end
   end
end

local function OnTrigger(uuid, t, m)
   if uuid == REPAIR_UUID then
      if m < 0 then
         m = math.abs(m)
         _G.WOWMSTracker[realm].AllChars[t].Spent = _G.WOWMSTracker[realm].AllChars[t].Spent + m
         _G.WOWMSTracker[realm][player][t].Spent = _G.WOWMSTracker[realm][player][t].Spent + m
      else
         addon.debugError("ERROR - Repair that we earned money from?!")
      end
   else
      if m > 0 then
         _G.WOWMSTracker[realm].AllChars[t].Earned = _G.WOWMSTracker[realm].AllChars[t].Earned + m
         _G.WOWMSTracker[realm][player][t].Earned = _G.WOWMSTracker[realm][player][t].Earned + m
      else
         m = math.abs(m)
         _G.WOWMSTracker[realm].AllChars[t].Spent = _G.WOWMSTracker[realm].AllChars[t].Spent + m
         _G.WOWMSTracker[realm][player][t].Spent = _G.WOWMSTracker[realm][player][t].Spent + m
      end
   end
end

mt.RegisterTracker(MERCH_UUID, 'MERCH', 'MERCHANT_SHOW', 'MERCHANT_CLOSED', OnTrigger)
mt.RegisterTracker(REPAIR_UUID, 'REPAIR', 'MERCHANT_SHOW', 'MERCHANT_CLOSED', OnTrigger)
mt.RegisterOnActivation(MERCH_UUID, OnActivation)
mt.RegisterDeterminant(REPAIR_UUID, 'UPDATE_INVENTORY_DURABILITY', Classify)
mt.RegisterDeterminant(MERCH_UUID, 'PLAYER_MONEY', Classify)
--mt.RegisterDeterminant(MERCH_UUID, 'MERCHANT_UPDATE', Classify) -- MERCHANT_UPDATE does not always trigger after change in player money

--TODO: Make a Grey / Green / Blue / Purple items catagory?
--TODO: Food / reagents / cooking supplies / ...
